/*
   Structs.cc
   A sample dBL Custom Class
   The dBASE Book
   by Kenneth J. Mayer. 
   Program copyright 2005-2007. 
   Permission granted for use as a learning tool for dBASE Plus.

   Sample from: Chapter 23 
   This encapsulates many useful methods for working with structures 
   and the Windows API, using dBL code.
*/

// Structs.CC
CLASS StrBaseStruct(nSize) of String

   this.sizeOf =              0

   if not empty(nSize)
      this.zero(nSize)  // Structure size = nSize
   else
      this.zero(2048)   // Default structure size to 2k
   endIf

   // PROTECTed stock properties
   PROTECT anchor, big, blink, bold, fixed, fontcolor,;
           fontsize, italics, link, small, strike, sup
   // PROTECTed stock methods
   PROTECT asc, charAt, chr, indexOf, isAlpha, isLower, isUpper,;
           lastIndexOf, left, leftTrim, replicate, right, rightTrim,;
           space, stuff, sub, subString, toLowerCase, toProperCase,;
           toUpperCase

   PROCEDURE Zero(nLen)

      if pCount() # 0
         this.sizeOf := nLen
      endIf
      this.string := replicate(chr(0), ceiling(this.sizeOf / 2))

   FUNCTION GetU16(nIndex)

      RETURN int(this.getByte(nIndex) +;
                 bitLShift(this.getByte(nIndex + 1), 8))

   FUNCTION Get16(nIndex)
      LOCAL n

      n = this.getU16(nIndex)
      RETURN iIf(n < 0x8000, n, -0x10000 + n)

   FUNCTION GetU32(nIndex)

      RETURN int(this.getByte(nIndex) +;
                 bitLShift(this.getByte(nIndex + 1),  8) +;
                 bitLShift(this.getByte(nIndex + 2), 16) +;
                 bitLShift(this.getByte(nIndex + 3), 24))

   FUNCTION Get32(nIndex)
      LOCAL n

      n = this.getU32(nIndex)
      RETURN iIf(n < 0x80000000, n, -0x100000000 + n)

   PROCEDURE SetU16(nIndex, nVal)

      this.setByte(nIndex, bitAnd(nVal, 255))
      this.setByte(nIndex + 1, bitAnd(bitRShift(nVal,  8), 255))

   PROCEDURE Set16(nIndex, nVal)

      this.setU16(nIndex, iIf(nVal >= 0, nVal, 0x10000 + nVal))

   PROCEDURE SetU32(nIndex, nVal)

      this.setByte(nIndex, bitAnd(nVal, 255))
      this.setByte(nIndex + 1, bitAnd(bitRShift(nVal,  8), 255))
      this.setByte(nIndex + 2, bitAnd(bitRShift(nVal, 16), 255))
      this.setByte(nIndex + 3, bitAnd(bitRShift(nVal, 24), 255))

   PROCEDURE Set32(nIndex, nVal)

      this.setU32(nIndex, iIf(nVal >= 0, nVal, 0x100000000 + nVal))

   PROCEDURE Logout(lHex)

      ? "---------------- Begin Logout ----------------"
      ? "Structure: " + this.ClassName +;
        " - Size: " + int(this.sizeOf) +;
                      "/0x" + iToH(this.sizeOf, 4)
      if type("this.logoutData") $ "CB|FP"
         this.logoutData()
      else
         ? "Formatted member data logout is not available."
      endIf
      if lHex
         this.logoutHex(this.sizeOf)
      endIf
      ? "----------------- End Logout -----------------"

   PROCEDURE LogoutHex(nLen)
      LOCAL n, x, xStr, cAscStr, nChr

      ? "-------------------------- Begin Dump"
      ?? " ------------------------------------"
      ? "       00 01 02 03 04 05 06 07  08 09 0A 0B 0C 0D 0E 0F"
      ?? "  01234567 89ABCDEF"
      ? "     --------------------------------------------------"
      ? "-------------------"
      ? " 0000|"
      n = iIf(pCount() # 0, nLen, this.length * 2)
      x = 1
      xStr = this.string
      cAscStr = "  "
      do
         nChr = this.getByte(x - 1)
         cAscStr += iIf((nChr < 32) or (nChr > 127), ".", chr(nChr))
         ?? " " + iToH(nChr, 2)
         if x % 8 = 0
            ?? " "
            cAscStr += " "
         endIf
         if (x % 16 = 0) or (x >= n)
            ?? cAscStr At 55
            if x < n
               ? " " + iToH(x, 4) + "|"
            endIf
            cAscStr = "  "
         endIf
      until ++ x > n
      ? "--------------------------- End Dump"
      ?? " -------------------------------------"

ENDCLASS  // StrBaseStruct of String

CLASS strRECT of StrBaseStruct(16)
 
   FUNCTION left(n)

      if pCount() > 0
         this.set32(0, n)
      endIf
      RETURN this.get32(0)                                  

   FUNCTION top(n)

      if pCount() > 0
         this.set32(4, n)
      endIf
      RETURN this.get32(4)

   FUNCTION right(n)

      if pCount() > 0
         this.set32(8, n)
      endIf
      RETURN this.get32(8)

   FUNCTION bottom(n)

      if pCount() > 0
         this.set32(12, n)
      endIf
      RETURN this.get32(12)

   PROCEDURE LogoutData

      ? "left:   " + "0x" + iToH(this.left(), 8)
      ? "top:    " + "0x" + iToH(this.top(), 8)
      ? "right:  " + "0x" + iToH(this.right(), 8)
      ? "bottom: " + "0x" + iToH(this.bottom(), 8)

ENDCLASS  // strRECT of StrBaseStruct

// EOF: Structs.CC
