; -- Mabie Brothers Inventory/Invoice System Install Script
;    for Inno Setup ...
;
;    This installer is based on one Lane Coddington did, allowing for
;    multiple setup options in the same installer.
;
;    These are:
;      Network Install -- create folders for DEO objects, tables, etc.
;         Install the ImportData program with runtime, BDE, etc.
;      Workstation Install -- install application, bde, runtime
;         Write folder locations for DEO into the local ini file?
;
;   In addition, this script uses code provided by Jonny Kwekkeboom
;   to find the network directory, so that we can fill in the DEO settings
;   of the application .INI folder for the Workstation Install.
;
; This script is being included in the sample code for The dBASE Book
; by Ken Mayer, as an example of a Network/DEO installation. If you examine
; it carefully you will see a section called "Types", one called "Components",
; and another called "Code". The "Components" flag is used in several places
; to determine if a user chooses to install to the file server or to a
; local workstation, the appropriate files. This is here as a courtesy to the
; readers of my book -- you cannot build an installer from this directly,
; but you can copy and paste from this script if you need to.

[Setup]
; If left at the default, this will show the components
; to be installed to the user, which could confuse them.
AlwaysShowComponentsList=no
; Used in the uninstall log:
AppID=Mabie Brothers Inventory and Invoice System
; Required - used to display the name of the application
; in several locations throughout the install.
AppName=Mabie Brothers Inventory and Invoice System
; These three are optional, but are displayed in the
; Windows 2000/XP/2003 (Vista?) Add/Remove Program
; applet ...
AppPublisher=Golden Stag Productions
AppPublisherURL=http://www.goldenstag.net/GSP
AppVersion=1.4
; Required: Should be the same text as AppName but include version
; number ...
AppVerName=Mabie Brothers Inventory and Invoice System version 1.4
; Required: displays default install location. The
; constant {sd} is the system drive.
DefaultDirName={sd}\MBIInvoice
; Name of the group in the Programs menu of Windows.
DefaultGroupName=MBI Inventory and Invoice System
; Defaults to "auto", if the folder installing to already
; exists, it asks if the Installer should overwrite the folder.
DirExistsWarning=no
; Disables the question about "This will install ...
; Do you want to continue?"
DisableStartupPrompt=yes
; Used on XP or Vista for installation rights.
PrivilegesRequired=Admin
; Where is the default source for the application?
; This information is used in the FILES section below.
SourceDir=C:\GSPCode\MabieBrothers\
; Output directory -- where the install program
; gets created (this will be "under" the source, i.e,
; C:\MyAppSource\InstallImage):
OutputDir=InstallImage
; Name of program to run (name of installer program)
OutputBaseFilename=MBISetup

; Appearance of the Installer:
SetupIconFile="InstallerImages\gears.ico"
WizardImageBackColor=clGreen
WizardImageFile="InstallerImages\SmallStag.bmp"
WizardImageStretch=no

; -----------------------------------------------------------------------------------------------------
; Define the two types:
[Types]
Name: Network; Description: Network Install -- Data and program files to network drive, and ImportData program to local
Name: Workstation; Description: Workstation Install -- for local workstation, installs Invoice program and needed files

; -----------------------------------------------------------------------------------------------------
; Components go with the types:
[Components]
Name: Net; Types: Network; Description: Network Install
Name: WS;  Types: Workstation; Description: Workstation Install

; -----------------------------------------------------------------------------------------------------
; Set up the directories on the network:
; Modified to do allow users to modify (which deals with editing/creating new/deleting/etc.)
; for tables and whatever other files are in the folders. This is done with the
;     Permissions: everyone-full;
; option. This may become more important for Windows Vista.
[Dirs]
Name: "{app}\Main"; Components: Net; Permissions: everyone-readexec;
Name: "{app}\dUFLP"; Components: Net; Permissions: everyone-readexec;
Name: "{app}\Other"; Components: Net; Permissions: everyone-readexec;
Name: "{app}\Images"; Components: Net; Permissions: everyone-readexec;
Name: "{app}\Tables"; Components: Net; Permissions: everyone-full;
; Where to install the dBASE Plus Runtime and the BDE:
Name: "{pf}\dBASE\Plus\Runtime"; Permissions: everyone-readexec; Flags: uninsneveruninstall;
Name: "{cf}\Borland Shared\BDE"; Permissions: everyone-full; Flags: uninsneveruninstall;

; -----------------------------------------------------------------------------------------------------
[Files]
; Files to be installed.

; Network install:
; None of these files in the network install should be uninstalled if that option is run --
; the Workstation application relies on these existing.
Source: "dUFLP\*.*"; DestDir: "{app}\dUFLP"; Components: Net; Flags: ignoreversion uninsneveruninstall
Source: "Main\*.*"; DestDir: "{app}\Main"; Components: Net; Flags: ignoreversion uninsneveruninstall
Source: "Other\*.*"; DestDir: "{app}\Other"; Components: Net; Flags: ignoreversion uninsneveruninstall
Source: "Images\*.*"; DestDir: "{app}\Images"; Components: Net; Flags: ignoreversion uninsneveruninstall
; Tables -- you don't want to overwrite them if they exist ... Also don't ever uninstall ...
Source: "EmptyTablesForInstall\*.*"; DestDir: "{app}\Tables"; Components: Net; Flags: onlyifdoesntexist uninsneveruninstall
;
; Needed for the ImportData routine, NET only:
Source: "ImportData.exe"; DestDir: "{app}"; Components: Net; Flags: ignoreversion
Source: "Install INI Files\ImportData.ini"; DestDir: "{app}"; Components: Net; Flags: ignoreversion
;
; Workstations:
Source: "MBIInvoice.exe"; DestDir: "{app}"; Components: WS; Flags: ignoreversion
Source: "Install INI Files\MBIInvoice.ini"; DestDir: "{app}"; Components: WS; Flags: ignoreversion

; all machines:
Source: "GSPRES32.DLL"; DestDir: "{app}"; Flags: ignoreversion

; dBASE Runtime Installer
Source: "C:\Program Files\dBASE\Plus\Runtime\PlusRuntime-b2119_EN.exe"; DestDir: "{tmp}"; Flags: ignoreversion deleteafterinstall
; this will be run to ensure permissions for the BDE ...
Source: "C:\Software Patches and Updates\dBASE\setacl.exe"; DestDir: {tmp}; Flags: deleteafterinstall; MinVersion: 0,4.0.1381

; Windows Vista Manifest Files
Source: "C:\Program Files\dBASE\PLUS\VistaManifest\Administrator\plusrun.exe.manifest"; DestDir: "{pf}\dBASE\Plus\Runtime"
Source: "C:\Program Files\dBASE\PLUS\VistaManifest\Administrator\bdeadmin.exe.manifest"; DestDir: "{cf}\Borland Shared\BDE"

; -----------------------------------------------------------------------------------------------------
[Run]
; Silent install of runtime suppresses the runtime install display, and the installshield stuff ...
; PlusRuntime-bNNNN_en.exe -s -a Runtime=d:\dBASEPLUS\Runtime BDE=d:\dBASEPLUS\bde Silent -sr
; In order to get this to work properly, the double-quotes are doubled up in a few places
; if you look at the statement ... that embeds a single double-quote (as it were) in the
; parameters needed for the runtime and bde installs.
Filename: {tmp}\PlusRuntime-b2119_EN.exe; Parameters: "-s -a Runtime=""{pf}\dBASE\Plus\Runtime"" BDE=""{cf}\Borland Shared\BDE"" Silent -sr"; StatusMsg: "Installing dBASE Runtime and BDE...";

; setAcl is used to give users full access to the registry keys at HKLM\SOFTWARE\Borland
Filename: {tmp}\setacl.exe; Parameters: MACHINE\SOFTWARE\Borland /registry /grant S-1-1-0 /set_val /sid; WorkingDir: {tmp}; MinVersion: 0,4.0.1381

; -----------------------------------------------------------------------------------------------------
[INI]
; Write/modify the INI file for the appropriate part of the app:
;
; ImportData first:
Filename: "{app}\ImportData.ini"; Section: "ObjectPath"; Key: "objPath0"; String: "{app}\Main"; Components: Net
Filename: "{app}\ImportData.ini"; Section: "ObjectPath"; Key: "objPath1"; String: "{app}\dUFLP"; Components: Net
Filename: "{app}\ImportData.ini"; Section: "ObjectPath"; Key: "objPath2"; String: "{app}\Other"; Components: Net
Filename: "{app}\ImportData.ini"; Section: "ObjectPath"; Key: "objPath3"; String: "{app}\Images"; Components: Net
;
; Main app ... using the code section to get the application folder on the network
Filename: "{app}\MBIInvoice.ini"; Section: "ObjectPath"; Key: "objPath0"; String: "{code:GetNetDir}\Main"; Components: WS
Filename: "{app}\MBIInvoice.ini"; Section: "ObjectPath"; Key: "objPath1"; String: "{code:GetNetDir}\dUFLP"; Components: WS
Filename: "{app}\MBIInvoice.ini"; Section: "ObjectPath"; Key: "objPath2"; String: "{code:GetNetDir}\Other"; Components: WS
Filename: "{app}\MBIInvoice.ini"; Section: "ObjectPath"; Key: "objPath3"; String: "{code:GetNetDir}\Images"; Components: WS

; -----------------------------------------------------------------------------------------------------
; Add icons ...
[Icons]
Name: "{group}\MBI Inventory and Invoice"; Filename: "{app}\MBIInvoice.exe"; WorkingDir: "{app}"; Components: WS
Name: "{group}\Import Data"; Filename: "{app}\ImportData.exe"; WorkingDir: "{app}"; Components: Net
; BDEAdmin ...
Name: "{group}\BDE Administrator"; Filename: "{cf}\Borland Shared\BDE\BDEAdmin.exe"; WorkingDir: "{cf}\Borland Shared\BDE"
; Desktop icons:
Name: "{userdesktop}\MBI Inventory and Invoice"; Filename: "{app}\MBIInvoice.exe"; WorkingDir: "{app}"; Components: WS
Name: "{userdesktop}\Import Data"; Filename: "{app}\ImportData.exe"; WorkingDir: "{app}"; Components: Net

; -----------------------------------------------------------------------------------------------------
; here's the important part -- we need to try to get the
; path to the files on the network here ...
; This code courtesy of Jonny Kwekkeboom
[Code]
var
  NetDirPage: TInputDirWizardPage;
  cNetDirDefault: String;

procedure InitializeWizard_NetDir;
var cTitle, cLabel, cDesc:String;
begin
  { Create the pages }

  cNetDirDefault := '{app}';
  cTitle := 'Select Directory on Network Drive'
  cLabel := 'Choose directory on the Network Drive where application files are installed.'
  cDesc  := 'Choose the folder on the network drive that has the subfolders "dUFLP", "Other", "Tables", etc. - this is most likely "Y:\MBIInvoice"'
  NetDirPage := CreateInputDirPage( wpSelectDir, cTitle, cLabel, cDesc, False, '' );
  NetDirPage.add('');
  NetDirPage.Values[0] := '';

end;

function GetNetDir( Param: String): String;
begin
   Result := NetDirPage.Values[0];
end;

procedure CurPageChanged( CurPageID: Integer);
begin
   Case CurPageID of
        NetDirPage.ID: begin
              NetDirPage.Values[0] := ExpandConstant( cNetDirDefault );
              if NetDirPage.Values[0] = '' then
                 NetDirPage.Values[0] := ExpandConstant( cNetDirDefault );
              end;
        end;
 end;

 procedure InitializeWizard();
 begin
    InitializeWizard_NetDir;
 end;

